#include "ps2_controller.h"

#include <kernel/portio.h>

static bool _keyboard_status = false;

void ps2_poll_output()
{
	while (!(inb(PS2_CONTROLLER_PORT) & 1));
}

void ps2_poll_input()
{
	while (inb(PS2_CONTROLLER_PORT) & 2);
}

// récupérer le statut du controleur
uint8_t kybrd_ctrl_read_status () {
	return(inb(PS2_CONTROLLER_PORT));
}

// récupérer le statut de l'encodeur
uint8_t kybrd_encd_read_status () {
	return(inb(PS2_ENCODER_PORT));
}

// envoyer une commande au controleur
void kybrd_ctrl_send_cmd(uint8_t cmd) {
	uint8_t status;

	do {
		// on vérifie que le controleur peut prendre une commande
		status = kybrd_ctrl_read_status();
	} while ((status & KYBRD_CTRL_STATS_MASK_IN_BUF) == 1);
	// on envoie au controleur
	outb(PS2_CONTROLLER_PORT, cmd);
}

// envoyer une commande à l'encodeur
void kybrd_encd_send_cmd(uint8_t cmd) {
	uint8_t status;

	do {
		// on vérifie que le controleur peut prendre une commande
		status = kybrd_ctrl_read_status();
	} while ((status & KYBRD_CTRL_STATS_MASK_IN_BUF) == 1);
	// on envoie à l'encodeur ATTENTION via le ctrl donc on doit faire la boucle d'au dessus
	outb(PS2_ENCODER_PORT, cmd);
}

// -- COMMANDES VERS ENCODEUR --

// set leds
void kybrd_set_leds(bool scroll, bool num, bool caps) {
	uint8_t data = 0;
	data = (scroll) ? (data | 1) : (data & 1);
	data = (num) ? (data | 2) : (data & 2);
	data = (caps) ? (data | 3) : (data & 3);

	kybrd_encd_send_cmd(KYBRD_ENC_CMD_SET_LED);
	kybrd_encd_send_cmd(data);
}

// 0 : renvoie le sc actuel sur 0x60, 1,2,3 : set le sc correspondant
void kybrd_set_scan_code(int scan_code_command) {
	uint8_t data = 0;
	switch (scan_code_command) {
		case 0:
			data = data | 1;
			break;
		case 1:
			data = data | 2;
			break;
		case 2:
			data = data | 4;
			break;
		case 3:
			data = data | 8;
			break;
	}

	kybrd_encd_send_cmd(KYBRD_ENC_CMD_SET_SCC);
	kybrd_encd_send_cmd(data);
}
// rpt_rate : 0 : 30 char/sec -> 31 : 2 chars/sec
// rpt_delay : 1/4 -> 1 sec par bit
void kybrd_set_repeat(uint8_t rpt_rate, uint8_t rpt_delay) {
	uint8_t data = 0;
	data = (data | rpt_rate) | (rpt_delay << 5);

	kybrd_encd_send_cmd(KYBRD_ENC_CMD_SET_RPT);
	kybrd_encd_send_cmd(data);
}

// send command to encoder
void kybrd_encd_generic_cmd(uint8_t cmd_code) {
	kybrd_encd_send_cmd(cmd_code);
}

// send command to a specific key encoder
void kybrd_spec_autorls(uint8_t cmd_code, uint8_t scan_code) {
	kybrd_encd_send_cmd(cmd_code);
	kybrd_encd_send_cmd(scan_code);
}

// -- COMMANDES VERS CONTROLEUR --

bool kybrd_self_test_ctrl() {
	kybrd_ctrl_send_cmd(KYBRD_CTRL_CMD_SLF_TST);

	while(1)
		if (kybrd_ctrl_read_status() & KYBRD_CTRL_STATS_MASK_OUT_BUF)
			break;

	// 0x55 self test passed 
	return (kybrd_encd_read_status() == 0x55) ? true : false;
}

void kybrd_disable_ctrl() {
	kybrd_ctrl_send_cmd(KYBRD_CTRL_CMD_DIS);
	_keyboard_status = false;
}


void kybrd_enable_ctrl() {
	kybrd_ctrl_send_cmd(KYBRD_CTRL_CMD_ENB);
	_keyboard_status = true;
}
