/*

On envoie des commandes (de 0xED à 0xFF (avec 0xF7 -> 0xFD inclu pour scan code 3 
seulement) au clavier et il répond (ACK 0xFA ou Resend 0xFE ou 0xEE Echo et des tests 
variés pour 0xFF).

On doit envoyer une commande à la fois et être prêt à Resend.

File de commande : ACK on pop cmd et on passe à la suivante, Resend on recommence.

Etat de machine : changement d'état quand des commandes s'exécutent avec succès.

3 scan code le défault c'est 2 celui que je vais essayer d'implémenter ici.

Ce sont des séquences de 1 ou plusieurs octets. Faudrait faire une table pour recensser 
les séquences possibles d'octets.

Pour les code de touche (key codes), y a rien de prédéterminé mais on devrait s'inspirer 
de trucs déjà existants et extensibles à d'autres périphériques comme la souris.

Quand l'état machine sait qu'il a reçu un scan code complet, on transforme ce scan 
code en key code.

Beaucoup d'informations sur la gestion des données dans la page "8042"_PS/2_Controller 
du wiki.
Les ports IO sont : 
0x60 | Read/Write | Data
0x64 | Read       | Status
0x64 | Write      | Command

0x60 : données reçues depuis clavier et écrire données vers le clavier

---
Broken Thorn : 

Keyb encoder, dans le kb qui envoie les scan codes
Keyb controller, dans la carte mère qui fait l'interface I/O
ON DOIT COMMUNIQUER AVEC LES DEUX !!!

2 types de scan code uniques pour chaque touche : Make (touche appuyé ou maintenue) 
Break (touche relachée) -> ensemble constitue le Scan Code Set

Les scan code sont sauvegardés dans le keyb controller

Types d'interface : 
- Polling : checker si une touche est appuyée ou pas avec le kb controller
- Interrupt Request : on peut faire en sorte que le kb controller nous envoie une interrupt 
a chaque fois qu'une touche est appuyé ou pas

0x60 sert pour accéder à l'encoder alors que 0x64 au controller.
Write envoie une commande
Read lit le buffer (0x60) ou le registre de statut (0x64)

Registre de statut (0x64) 8 bits : 

0: Output buffer status : 0 -> empty ; 1 -> full (prêt à lire)
1: Input buffer status : 0 -> empty (prêt pour écrire) ; 1 -> full 
2: System flag (1 si test ok)
3: Command data : 0 -> dernier write sur 0x60 ; 1 -> dernier write sur 0x64
4: Keyboard Locked (1 si vérouillé)
5: Auxiliary buffer full 
6: Timeout 
7: Parity error

Quand on envoie une commande à 0x60 on doit s'assurer que le controller est prêt 
à recevoir étant donné que la commande va aussi passé par le controller.

Commandes pour le controller remarquables : 
- 0x20 : Lecture du byte de commande (et la RAM pcq c'est la même chose, le byte de 
commande est dans la RAM) (pas utile)
- 0x60 : Ecriture byte de commande (pas utile dans la démo)
- cf l'enum en question

*/

#ifndef PS2_CONTROLLER_H
#define PS2_CONTROLLER_H 1

#include <stdint.h>
#include <stdbool.h>

#ifdef __arch_i386__
#define PS2_ENCODER_PORT	0x60
#define PS2_CONTROLLER_PORT	0x64
#endif

typedef enum _ps2_status_register {
	KYBRD_CTRL_STATS_MASK_OUT_BUF	=	1,		  //00000001
	KYBRD_CTRL_STATS_MASK_IN_BUF	=	2,		  //00000010
	KYBRD_CTRL_STATS_MASK_SYSTEM	=	4,		  //00000100
	KYBRD_CTRL_STATS_MASK_CMD_DATA	=	8,		//00001000
	KYBRD_CTRL_STATS_MASK_LOCKED	=	0x10,		//00010000
	KYBRD_CTRL_STATS_MASK_AUX_BUF	=	0x20,		//00100000
	KYBRD_CTRL_STATS_MASK_TIMEOUT	=	0x40,		//01000000
	KYBRD_CTRL_STATS_MASK_PARITY	=	0x80		//10000000
} ps2_status_register;

// commandes pour l'encodeur
enum KYBRD_ENC_CMD {
	KYBRD_ENC_CMD_SET_LED         = 0xED, // set leds
	KYBRD_ENC_CMD_ECHO            = 0xEE, // echo command
	KYBRD_ENC_CMD_SET_SCC         = 0xF0, // set alternate scan code
	KYBRD_ENC_CMD_ID              = 0xF2, // demande id (lue sur 0x60)
	KYBRD_ENC_CMD_SET_RPT         = 0xF3, // set autorepeat delay & repeat rate
	KYBRD_ENC_CMD_ENB             = 0xF4, // enable kbd
	KYBRD_ENC_CMD_DIS             = 0xF5, // disable kbd + paramètres par défaut (parfois)
	KYBRD_ENC_CMD_DFT             = 0xF6, // met les paramètres par défaut du clavier
	KYBRD_ENC_CMD_AUTORPT         = 0xF7, // SC 3 seulement : tout le monde en autorepeat
	KYBRD_ENC_CMD_MKE_RLS         = 0xF8, // SC 3 seulement : tout le monde en make/release
	KYBRD_ENC_CMD_MKE             = 0xF9, // SC 3 seulement : tout le monde en make
	KYBRD_ENC_CMD_MKE_RLS_AUTORPT = 0xFA, // SC 3 seulement : tout le monde en te
	KYBRD_ENC_CMD_SPC_AUTORPT     = 0xFB, // SC 3 seulement : 1 en autorpt 
	KYBRD_ENC_CMD_SPC_MKE_RLS     = 0xFC, // SC 3 seulement : 1 en make/release 
	KYBRD_ENC_CMD_SPC_MKE             = 0xFD, // SC 3 seulement : 1 en make
	KYBRD_ENC_CMD_RSD             = 0xFE, // renvoie du dernier résultat
	KYBRD_ENC_CMD_RST_TEST        = 0xFF, // réinitialise et test
};

enum KYBRD_ENC_RET_CODE {
	KYBRD_ENC_RET_BUFF_OVF = 0x0,  // buffer overflow
	KYBRD_ENC_RET_BAT_OK   = 0xAA, // self test passed
	KYBRD_ENC_RET_ACK      = 0xFA, // acknowledged
	KYBRD_ENC_RET_BAT_FL_1 = 0xFC, // self test failed
	KYBRD_ENC_RET_BAT_FL_2 = 0xFD, // self test failed
	KYBRD_ENC_RET_RSD      = 0xFE, // resend
	KYBRD_ENC_RET_KEY_ERR  = 0xFF, // key error
};

enum KYBRD_CTRL_CMD {
	KYBRD_CTRL_CMD_SLF_TST  = 0xAA, // self test 0x55 sur 0x60 : success !
	KYBRD_CTRL_CMD_INTF_TST = 0xAB, // interface test
	KYBRD_CTRL_CMD_DIS      = 0xAD, // keyboard disable
	KYBRD_CTRL_CMD_ENB      = 0xAE, // keyboard enable
};

//-- Fonctions --
// lecture
void ps2_poll_output();
void ps2_poll_input();

// envoi de commandes
void kybrd_ctrl_send_cmd(uint8_t cmd);
void kybrd_enc_send_cmd(uint8_t cmd);

// commandes vers encodeur
void kybrd_set_leds(bool scroll, bool num, bool caps);
void kybrd_set_scan_code(int scan_code_command);
void kybrd_set_repeat(uint8_t rpt_rate, uint8_t rpt_delay);

void kybrd_encd_generic_cmd(uint8_t cmd_code);
void kybrd_encd_spec_key_cmd(uint8_t cmd_code, uint8_t scan_code);

// commandes vers controller
bool kybrd_self_test_ctrl();
void kybrd_disable_ctrl();
void kybrd_enable_ctrl();

#endif //PS2_CONTROLLER_H
