#include <kernel/tty.h>
#include <kernel/msg.h>
#include <kernel/keyboard.h>
#include <kernel/timer.h>

#define KBUFF_MAX_SIZE 20

char keybuffer[KBUFF_MAX_SIZE];
volatile unsigned int kbuff_front;
volatile unsigned int kbuff_back;
volatile unsigned int kbuff_size;

void reset_pending_keypress()
{
	kbuff_front = 0;
	kbuff_back = 0;
	kbuff_size = 0;
}

void add_pending_keypress(char keypress)
{
	if (kbuff_size == KBUFF_MAX_SIZE)
	{
		term_writestring("Discarded keypress\n");
		return;
	}

	keybuffer[kbuff_front] = keypress;
	kbuff_front = (kbuff_front + 1) % KBUFF_MAX_SIZE;
	kbuff_size++;
}

char pop_pending_keypress()
{
	if (kbuff_size == 0)
	{
		term_writestring("No pending keypress\n");
		return '\0';
	}

	char keypress = keybuffer[kbuff_back];
	kbuff_back = (kbuff_back + 1) % KBUFF_MAX_SIZE;
	kbuff_size--;
	return keypress;
}

bool is_pending_keypress()
{
	return (kbuff_size != 0);
}

extern kbd_driver ps2_kbd_driver;
extern timer_driver pit_driver;

void hardware_init()
{
	term_init();
	pit_driver.base.driver_init();
	ps2_kbd_driver.base.driver_init();
}

void kernel_main()
{
	term_writestring("Vive Thomazo !\n");
	keyboard *ps2_kbd = ps2_kbd_driver.add_keyboard("");

	reset_pending_keypress();
	term_writestring("Reset keypresses\n");
	term_putchar('\n');
	while (1)
	{
		ps2_kbd_driver.base.driver_update();
		if (is_pending_keypress())
		{
			uint8_t sc = ps2_kbd_driver.code_to_key(ps2_kbd, pop_pending_keypress());
			if (sc)
				term_putchar(sc);
		}
	}
}
